bl_info = {
    "name": "Colorbox",
    "author": "xz-blender,Essential Elephant, Jos Vromans, Leandro Benedet Garcia",
    "version": (2, 0, 1),
    "blender": (2, 93, 0),
    "location": "UI -> Colorbox",
    "description": "Colorbox",
    "warning": "",
    "wiki_url": "https://github.com/Essential-Elephant/",
    "tracker_url": "https://github.com/Essential-Elephant/",
    "category": "",
}

ADDON_ID = __package__
first_startup = "bpy" not in locals()
import bpy

from . import constants
from . import material_mixer
from . import operators
from . import preferences
from . import properties
from . import ui
from . import utils
from . import translate

if not first_startup:
    import importlib
    importlib.reload(constants)
    importlib.reload(material_mixer)
    importlib.reload(operators)
    importlib.reload(preferences)
    importlib.reload(properties)
    importlib.reload(ui)
    importlib.reload(utils)


blender_classes = [
    ui.VIEW3D_PT_swap_panel,
    ui.VIEW3D_PT_color_box,
    ui.VIEW3D_PT_color_ramps,
    ui.VIEW3D_PT_color_schemes,
    ui.NODE_EDITOR_PT_swap_panel,
    ui.NODE_EDITOR_PT_color_box,
    ui.NODE_EDITOR_PT_color_ramps,
    ui.NODE_EDITOR_PT_color_schemes,
    ui.IMAGE_EDITOR_PT_color_box,
    ui.IMAGE_EDITOR_PT_color_ramps,
    ui.IMAGE_EDITOR_PT_color_schemes,

    # preferences
    preferences.ColorBoxPreferences,
    # Properties
    properties.WeightedColor,
    properties.Palette,
    properties.PaletteCollection,
    properties.ColorRamp,
    properties.ColorRampCollection,
    properties.ColorBoxOptions,
    # Operators
    operators.COLORBOX_OT_add_row,
    operators.COLORBOX_OT_remove_row,
    operators.COLORBOX_OT_add_color,
    operators.COLORBOX_OT_remove_color,
    operators.COLORBOX_OT_create_nodes,
    operators.COLORBOX_OT_next_palette_page,
    operators.COLORBOX_OT_previous_palette_page,
    operators.COLORBOX_OT_load_json,
    operators.COLORBOX_OT_save_json,
    operators.COLORBOX_OT_import_blender_palettes,
    operators.COLORBOX_OT_reverse_colors,
    operators.COLORBOX_OT_apply_gamma,
    operators.COLORBOX_OT_apply_inverse_gamma,
    operators.COLORBOX_OT_copy_color_row,
    operators.COLORBOX_OT_sort_by_hue,
    operators.COLORBOX_OT_average_color_rgb,
    operators.COLORBOX_OT_average_color_hsv,
    operators.COLORBOX_OT_print_colors,
    operators.COLORBOX_OT_add_to_color_ramp,
    operators.COLORBOX_OT_extract_from_image,
    operators.COLORBOX_OT_extract_from_palette,
    operators.COLORBOX_OT_export_png,
    operators.COLORBOX_OT_export_png_collection,
    operators.COLORBOX_OT_palette_to_swapper,
    operators.COLORBOX_OT_collection_add,
    operators.COLORBOX_OT_collection_remove,
    operators.COLORBOX_OT_collection_show,
    operators.COLORBOX_OT_collection_duplicate,
    operators.COLORBOX_OT_collection_reverse,
    operators.COLORBOX_OT_color_ramp_add,
    operators.COLORBOX_OT_color_ramp_remove,
    operators.COLORBOX_OT_color_ramp_save,
    operators.COLORBOX_OT_color_ramp_load,
    operators.COLORBOX_OT_color_ramp_duplicate,
    operators.COLORBOX_OT_color_ramp_convert_to_palette,
    operators.COLORBOX_OT_create_swapper_nodes,
    operators.color_scheme.COLORBOX_OT_color_scheme_help,
    operators.color_scheme.COLORBOX_OT_color_scheme_add_to_collection,
    # menu's
    operators.COLORBOX_OT_show_menu,
    operators.PANEL_PT_MoreOptions,
    # modals
    operators.COLORBOX_OT_modal_color_picker,
    # Color Swapper
    operators.COLORBOX_OT_add_to_color_swapper,
    operators.WM_MT_button_context,
    properties.ColorFieldPointProperty,
    properties.ColorPointersCollection,
    properties.ColorSwapCollection,
    operators.COLORBOX_OT_swap_collection_add_multiple,
    operators.COLORBOX_OT_swap_collection_add,
    operators.COLORBOX_OT_clear_all_swappers,
    operators.COLORBOX_OT_swap_collection_delete,
    operators.COLORBOX_OT_swap_color_pointer_delete,
    ui.COLORBOX_UL_swap_collection,
    ui.COLORBOX_UL_swap_color,
    operators.COLORBOX_OT_swap_colorbox_navigation,
    operators.COLORBOX_OT_swap_apply_palette,
    operators.COLORBOX_OT_swap_shuffle,
]


def draw_node_menu(self, context):
    """
    Only when there is (at least) one color ramp node selected, draw the menu.
    In all other cases, do nothing.
    """
    if any(
        isinstance(node, bpy.types.ShaderNodeValToRGB)
        for node in context.selected_nodes
    ):
        layout = self.layout
        layout.separator()
        layout.operator(
            "color_box.color_ramp_save",
            text="Add selected Color Ramp(s) to Colorbox",
        )


def context_menu_func(self, context):
    if operators.COLORBOX_OT_add_to_color_swapper.poll(context):
        layout = self.layout
        layout.separator()

        # this is a method that seems to be rarely used in general, but basically it executes
        # the operator after the user selects something in a drop down menu, the list of things
        # comes from a enumerator inside the operator, the second argument gets the attribute
        # with the same name of the string
        layout.operator_menu_enum(
            "object.add_to_color_swapper", "color_right_click_menu"
        )


addon_keymaps = []


def register():
    for blender_class in blender_classes:
        bpy.utils.register_class(blender_class)

    material_mixer.register()

    # Palette variables
    bpy.types.Scene.color_box = bpy.props.CollectionProperty(
        type=properties.PaletteCollection
    )
    bpy.types.Scene.color_box_palette_current_page = bpy.props.IntProperty(
        name="Current page", min=0
    )
    bpy.types.Scene.color_box_palette_page_amount = bpy.props.IntProperty(
        name="Total number of pages", min=0
    )
    bpy.types.Scene.color_box_palette_per_page = bpy.props.IntProperty(
        name="Palette per page", min=1, default=10
    )
    bpy.types.Scene.color_box_total_palettes = bpy.props.IntProperty(
        min=0, default=0
    )

    # Color ramp variables
    bpy.types.Scene.color_box_ramps = bpy.props.CollectionProperty(
        type=properties.ColorRampCollection
    )
    bpy.types.Scene.color_box_swapper = bpy.props.PointerProperty(
        type=properties.ColorSwapCollection
    )
    bpy.types.Scene.color_box_selected_menu = bpy.props.IntVectorProperty(
        size=3, default=(-1, -1, -1)
    )
    bpy.types.Scene.color_box_selected_row = bpy.props.IntVectorProperty(
        size=3, default=(-1, -1, -1)
    )
    bpy.types.Scene.color_box_options = bpy.props.PointerProperty(
        type=properties.ColorBoxOptions
    )
    bpy.types.NODE_MT_context_menu.append(draw_node_menu)
    bpy.types.WM_MT_button_context.append(context_menu_func)
    
    properties.register()

    wm = bpy.context.window_manager
    km = wm.keyconfigs.addon.keymaps.new(name="Window", space_type="EMPTY")
    kmi = km.keymap_items.new(
        "color_box.modal_color_picker", "P", "PRESS", shift=True, ctrl=True
    )
    addon_keymaps.append((km, kmi))
    translate.register()


def unregister():
    scene = bpy.context.scene

    for km, kmi in addon_keymaps:
        km.keymap_items.remove(kmi)
    addon_keymaps.clear()

    bpy.types.WM_MT_button_context.remove(context_menu_func)
    bpy.types.NODE_MT_context_menu.remove(draw_node_menu)

    scene.color_box.clear()
    scene.color_box_ramps.clear()
    scene.color_box_swapper.color_collection.clear()

    del bpy.types.Scene.color_box_palette_current_page
    del bpy.types.Scene.color_box_total_palettes
    del bpy.types.Scene.color_box_palette_page_amount
    del bpy.types.Scene.color_box_options
    del bpy.types.Scene.color_box_selected_row
    del bpy.types.Scene.color_box_selected_menu
    del bpy.types.Scene.color_box_swapper
    del bpy.types.Scene.color_box_ramps
    del bpy.types.Scene.color_box

    for blender_class in reversed(blender_classes):
        bpy.utils.unregister_class(blender_class)

    material_mixer.unregister()
    translate.unregister()
