import random

import bpy
from bpy.types import Operator


def clear_all_materials(context):
    for obj in context.selected_objects:
        obj.data.materials.clear()


class COLORBOX_MATERIALMIXER_OT_ClearMaterials(Operator):
    """Remove materials from all selected objects"""

    bl_idname = "color_box.material_mixer_clear_material"
    bl_label = "Clear Objects Materials"
    bl_description = "Clear materials from selected objects"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        clear_all_materials(context)
        return {"FINISHED"}


class COLORBOX_MATERIALMIXER_OT_ApplyMaterials(Operator):
    """Apply new materials to the selected objects"""

    bl_idname = "color_box.material_mixer_apply_materials"
    bl_label = "Apply materials"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        materials = [
            x for x in bpy.data.materials if not x.is_grease_pencil
        ]
        weights = [w.color_box_material_mixer_weight for w in materials]

        sel_objects = context.selected_objects
        number_of_selected_objects = len(sel_objects)

        clear_all_materials(context)

        chosen_mats = random.choices(
            materials, weights=weights, k=number_of_selected_objects
        )

        for curr_mat, curr_obj in zip(chosen_mats, sel_objects):
            curr_obj.data.materials.append(curr_mat)
        return {"FINISHED"}


class COLORBOX_MATERIALMIXER_OT_CreateNewMaterial(Operator):
    """Create a new material"""

    bl_idname = "color_box.material_mixer_add"
    bl_label = "Create New Mixer Material"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        bpy.data.materials.new("Material Mixer")
        return {"FINISHED"}


class COLORBOX_MATERIALMIXER_OT_RemoveNewMaterial(Operator):
    """Remove selected material"""

    bl_idname = "color_box.material_mixer_remove"
    bl_label = "Remove New Mixer Material"
    bl_options = {"REGISTER", "UNDO"}

    def execute(self, context):
        scene = context.scene
        materials = bpy.data.materials
        active_material = materials[scene.color_box_material_mixer_active]
        bpy.data.materials.remove(active_material)
        return {"FINISHED"}
