import bpy
from bpy.types import Context, Material, Panel, UILayout, UIList
from bpy_extras.node_utils import find_node_input


def panel_node_draw(layout, ntree, _output_type, input_name):
    node = ntree.get_output_node("EEVEE")

    if node:
        input = find_node_input(node, input_name)
        if input:
            layout.template_node_view(ntree, node, input)
        else:
            layout.label(text="Incompatible output node")
    else:
        layout.label(text="No output node")


class MATERIAL_UL_material_mixer(UIList):
    def draw_item(
        self,
        _context,
        layout: UILayout,
        _data,
        item: Material,
        icon,
        _active_data,
        _active_propname,
        _index,
    ):
        layout.context_pointer_set("id", item)

        if self.layout_type in {"DEFAULT", "COMPACT"}:
            if item:
                layout.prop(
                    item, "name", text="", emboss=False, icon_value=icon
                )
            else:
                layout.label(text="", icon_value=icon)

        elif self.layout_type == "GRID":
            layout.alignment = "CENTER"
            layout.label(text="", icon_value=icon)

        layout.prop(
            item, "color_box_material_mixer_weight", text="", slider=True
        )

    def filter_items(self, context, data, propname):
        helper_funcs = bpy.types.UI_UL_list
        items = getattr(data, propname)

        filtered = [self.bitflag_filter_item] * len(items)
        order = helper_funcs.sort_items_by_name(items, "name")

        for i, item in enumerate(items):
            if item.is_grease_pencil:
                filtered[i] &= ~self.bitflag_filter_item

        return filtered, order


class MaterialMixer:
    bl_region_type = "UI"
    bl_category = "Colorbox"
    bl_label = "MATERIAL MIXER"
    bl_space_type = "VIEW_3D"
    bl_options = {"DEFAULT_CLOSED"}


class VIEW_3D_PT_MaterialMixerPanel(MaterialMixer, Panel):
    def draw(self, context: Context):
        scene = context.scene
        layout = self.layout

        row = layout.row()
        row.operator("color_box.material_mixer_clear_material")
        row.operator("color_box.material_mixer_apply_materials")

        row = layout.row()
        row.template_list(
            "MATERIAL_UL_material_mixer",
            "",
            bpy.data,
            "materials",
            scene,
            "color_box_material_mixer_active",
            rows=3,
        )

        col = row.column(align=True)

        col.operator("color_box.material_mixer_add", icon="ADD", text="")

        col.operator("color_box.material_mixer_remove", icon="REMOVE", text="")

        col.separator()
        """
        col.menu("MATERIAL_MT_context_menu", icon='DOWNARROW_HLT', text="")
        """


class VIEW_3D_PT_MaterialMixerPreview(MaterialMixer, Panel):
    bl_label = "Preview"
    bl_parent_id = "VIEW_3D_PT_MaterialMixerPanel"
    bl_options = {"HEADER_LAYOUT_EXPAND"}

    @classmethod
    def poll(cls, context: Context):
        return (
            len(bpy.data.materials)
            > context.scene.color_box_material_mixer_active
        )

    def draw(self, context: Context):
        try:
            active_material = bpy.data.materials[
                context.scene.color_box_material_mixer_active
            ]
        except IndexError:
            active_material = False

        if active_material:
            self.layout.template_preview(active_material)


class VIEW_3D_PT_MaterialMixerEditMaterial(MaterialMixer, Panel):
    bl_label = "Edit Material"
    bl_parent_id = "VIEW_3D_PT_MaterialMixerPanel"
    bl_options = {"HEADER_LAYOUT_EXPAND"}

    @classmethod
    def poll(cls, context: Context):
        return (
            len(bpy.data.materials)
            > context.scene.color_box_material_mixer_active
        )

    def draw(self, context: Context):
        layout = self.layout

        try:
            active_material = bpy.data.materials[
                context.scene.color_box_material_mixer_active
            ]
        except IndexError:
            active_material = False

        if active_material:
            layout.prop(active_material, "use_nodes", icon="NODETREE")
            layout.separator()

            if active_material.use_nodes:
                layout.prop(
                    active_material,
                    "color_box_material_mixer_weight",
                    slider=True,
                )
                panel_node_draw(
                    self.layout,
                    active_material.node_tree,
                    "OUTPUT_MATERIAL",
                    "Surface",
                )
            else:
                layout.prop(
                    active_material,
                    "color_box_material_mixer_weight",
                    slider=True,
                )
                layout.prop(
                    active_material, "diffuse_color", text="Base Color"
                )
                layout.prop(active_material, "metallic")
                layout.prop(
                    active_material, "specular_intensity", text="Specular"
                )
                layout.prop(active_material, "roughness")
