
import bpy
from .. import utils
from . import color_swapper


class COLORBOX_OT_color_scheme_help(bpy.types.Operator):
    bl_idname = "color_box.color_scheme_help"
    bl_label = "Color Schemes Help"
    bl_description = "Show info about the different color schemes"
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    def execute(self, context):
        # Open in browser
        bpy.ops.wm.url_open(url="https://i.imgur.com/HQL7Y81.png")
        return {"FINISHED"}


class COLORBOX_OT_color_scheme_add_to_collection(bpy.types.Operator):
    bl_idname = "color_box.color_scheme_add_to_collection"
    bl_label = "Add to Palette Collection"
    bl_description = "Add this color scheme to the palette collection"
    bl_options = {'REGISTER', 'UNDO', 'INTERNAL'}

    def enum_collections(self, context):
        """EnumProperty callback"""
        enum_items = []
        
        # Add all collection items to the enum
        for i, item in enumerate(context.scene.color_box):
            enum_items.append((str(i), item.name, f"Add to collection {item.name}", "TRIA_RIGHT", len(enum_items)))
        
        # Add the option to add a new collection to the enum
        enum_items.append(("ADD", f"Add new collection", "Add a new collection", "ADD", len(enum_items)))

        return enum_items
    
    mode: bpy.props.EnumProperty(
        items=enum_collections,
    )
    new_collection_name: bpy.props.StringProperty(
        name="New Collection Name",
        default="New Collection",
    )

    def invoke(self, context, event):
        wm = context.window_manager
        return wm.invoke_props_dialog(self)
    
    def draw(self, context):
        self.layout.prop(self, "mode", text="")
        if self.mode == "ADD":
            self.layout.prop(self, "new_collection_name", text="Name")
        self.layout.separator()

    def execute(self, context):
        # Add new collection
        if self.mode == "ADD":
            new_collection = context.scene.color_box.add()
            new_collection.name = self.new_collection_name
            new_collection.show = True
            color_swapper.sync_color_slots(context.scene)
            collection_index = len(context.scene.color_box) - 1
        else:
            collection_index = int(self.mode)
        
        # Get all colors fro mthe scheme
        colors = [context.scene.color_box_scheme_color_1, context.scene.color_box_scheme_color_2, context.scene.color_box_scheme_color_3, context.scene.color_box_scheme_color_4]

        # Remove transparent colors
        for color in reversed(colors):
            if color[3] == 0:
                colors.remove(color)
        
        # Add new palette to the selected collection
        utils.move_rows.add_palette(
            context.scene,
            palette_name=context.scene.color_box_mode,
            colors=colors,
            collection_index=collection_index,
        )
        
        # Update UI
        color_swapper.sync_color_slots(context.scene)
        context.area.tag_redraw()
        
        self.report({"INFO"}, f"Color scheme added to palette collection")
        return {"FINISHED"}

