import bpy


def draw_collection_menu_palettes(self, context):
    layout = self.layout
    scene = context.scene

    _type, col_index, row_index = context.scene.color_box_selected_menu
    if not (col_index >= 0 and row_index == -1):
        return

    layout.label(text="More options")

    if (
        context.area.type == "IMAGE_EDITOR"
        and context.space_data.ui_mode == "VIEW"
    ):
        picker_op = layout.operator(
            "color_box.modal_color_picker",
            text="Launch a color picker",
            icon="EYEDROPPER",
        )
        picker_op.collection_index = col_index

    layout.prop(context.scene.color_box[col_index], "show_color_selection")
    layout.operator("color_box.collection_duplicate")
    layout.operator("color_box.collection_reverse")

    layout.separator()
    layout.label(text="Effects:")
    layout.prop(context.scene.color_box_options, "selected_effector", text="")
    layout.separator()

    layout.operator("color_box.export_png_collection", text="Generate .png")
    layout.operator("color_box.add_to_color_ramp", text="Make Color Ramps")


def draw_collection_menu_color_ramps(self, context):
    """
    - Add a new collection in Palettes, and add all color ramps as a palette

    - Add all color ramps from this collection to the material
    """
    _type, col_index, row_index = context.scene.color_box_selected_menu
    if not (col_index >= 0 and row_index == -1):
        return

    layout = self.layout
    layout.operator("color_box.collection_duplicate")
    layout.operator(
        "color_box.color_ramp_convert_to_color_row", text="Make Palettes"
    )


def draw_row_menu_palettes(self, context):
    layout = self.layout

    _type, col_index, row_index = context.scene.color_box_selected_menu
    if not (col_index >= 0 and row_index >= 0):
        return

    layout.label(text="More options")

    if (
        context.area.type == "IMAGE_EDITOR"
        and context.space_data.ui_mode == "VIEW"
    ):
        picker_op = layout.operator(
            "color_box.modal_color_picker",
            text="Launch a color picker",
            icon="EYEDROPPER",
        )
        picker_op.collection_index = col_index
        picker_op.row_index = row_index

    layout.prop(
        context.scene.color_box[col_index].palettes[row_index],
        "show_color_selection",
    )
    layout.operator("color_box.copy_row")
    layout.operator("color_box.export_png", text="Generate .png")
    layout.operator("colorbox.palette_to_swapper", text="Add to swapper")
    layout.operator("color_box.create_nodes")

    layout.separator()
    layout.label(text="Effects:")
    layout.prop(context.scene.color_box_options, "selected_effector", text="")
    layout.separator()

    col = layout.column()
    col.label(text="Convert this row into a Color Ramp")
    col.prop(context.scene.color_box_options, "selected_color_ramp", text="")


def draw_row_menu_color_ramps(self, context):
    layout = self.layout

    _type, col_index, row_index = context.scene.color_box_selected_menu
    if not (col_index >= 0 and row_index >= 0):
        return

    layout.operator("color_box.color_ramp_duplicate")

    layout.separator()
    col = layout.column()
    col.label(text="Convert this color ramp into a palette")
    col.prop(context.scene.color_box_options, "selected_palette", text="")


def draw_collection_menu(self, context):
    _type, col_index, row_index = context.scene.color_box_selected_menu

    if _type == 0:
        return draw_collection_menu_palettes(self, context)
    elif _type == 1:
        return draw_collection_menu_color_ramps(self, context)


def draw_row_menu(self, context):
    _type, col_index, row_index = context.scene.color_box_selected_menu

    if _type == 0:
        return draw_row_menu_palettes(self, context)
    elif _type == 1:
        return draw_row_menu_color_ramps(self, context)


class PANEL_PT_MoreOptions(bpy.types.Panel):
    bl_label = "More options"
    bl_space_type = "PROPERTIES"
    bl_region_type = "WINDOW"
    bl_context = (
        "0data"  # to hide it. bl_context = "object" to show it in props
    )
    bl_options = {"DEFAULT_CLOSED"}

    def draw(self, context):
        _type, col_index, row_index = context.scene.color_box_selected_menu

        if row_index == -1:
            draw_collection_menu(self, context)
        else:
            draw_row_menu(self, context)


class COLORBOX_OT_show_menu(bpy.types.Operator):
    bl_idname = "color_box.show_menu"
    bl_label = "More options"
    bl_options = {"REGISTER"}

    collection_type: bpy.props.IntProperty(default=0)
    collection_index: bpy.props.IntProperty(default=-1)
    row_index: bpy.props.IntProperty(default=-1)

    def execute(self, context):
        scene = context.scene
        scene.color_box_selected_menu = (
            self.collection_type,
            self.collection_index,
            self.row_index,
        )

        # better to set it always to the default, otherwise a non existing collection index could still be selected
        scene.color_box_options.selected_color_ramp = "-2"
        scene.color_box_options.selected_palette = "-2"

        bpy.ops.wm.call_panel(name="PANEL_PT_MoreOptions", keep_open=True)
        return {"FINISHED"}
